@@ -1,3 +1,5 @@
+/* $Id: procmap.c,v 1.22 2006/01/23 16:41:10 bdejean Exp $ */
+
 /* Copyright (C) 1998-99 Martin Baulig
    This file is part of LibGTop 1.0.
 
@@ -36,7 +38,9 @@
 #define SMAPS_FILE "/proc/%u/smaps"
 
 
-#define PROC_MAPS_FORMAT "%16" G_GINT64_MODIFIER "x-%16" G_GINT64_MODIFIER "x %4c %16" G_GINT64_MODIFIER "x %02hx:%02hx %" G_GINT64_MODIFIER "u%*[ ]%n"
+#define PROC_MAPS_FORMAT ((sizeof(void*) == 8) \
+? "%16lx-%16lx %4c %16lx %02hx:%02hx %lu%*[ ]%[^\n]\n" \
+: "%08lx-%08lx %4c %08lx %02hx:%02hx %lu%*[ ]%[^\n]\n")
 
 
 static const unsigned long _glibtop_sysdeps_proc_map =
@@ -58,7 +62,7 @@ static const unsigned long _glibtop_sysdeps_map_entry_smaps =
 /* Init function. */
 
 void
-_glibtop_init_proc_map_s (glibtop *server)
+glibtop_init_proc_map_s (glibtop *server)
 {
 	server->sysdeps.proc_map = _glibtop_sysdeps_proc_map;
 }
@@ -66,78 +70,49 @@ _glibtop_init_proc_map_s (glibtop *server)
 /* Provides detailed information about a process. */
 
 
-
-struct smap_value {
-	char name[16];
-	size_t name_len;
-	ptrdiff_t offset;
-};
-
-static int
-compare(const void* a_key, const void* a_smap)
-{
-	const char* key = a_key;
-	const struct smap_value* smap = a_smap;
-	return strncmp(key, smap->name, smap->name_len);
-}
-
-
-static gboolean
-is_smap_value(const char* s)
-{
-	for ( ; *s; ++s) {
-
-		if (isspace(*s))
-			return FALSE;
-
-		if (*s == ':')
-			return TRUE;
-	}
-
-	return FALSE;
-}
-
-
-/*
-  Returns whether line is a 'value' line
-  and add if we know its meaning
-*/
-static gboolean
-parse_smaps(glibtop_map_entry *entry, const char* line)
+static void
+add_smaps(glibtop *server, FILE *smaps, glibtop_map_entry *entry)
 {
 #define SMAP_OFFSET(MEMBER) offsetof(glibtop_map_entry, MEMBER)
-#define STR_AND_LEN(X) (X), (sizeof X - 1)
 
-	/* keep sorted */
-	const struct smap_value values[] = {
-		{ STR_AND_LEN("Private_Clean:"),	SMAP_OFFSET(private_clean) },
-		{ STR_AND_LEN("Private_Dirty:"),	SMAP_OFFSET(private_dirty) },
-		{ STR_AND_LEN("Rss:"),			SMAP_OFFSET(rss) },
-		{ STR_AND_LEN("Shared_Clean:"),		SMAP_OFFSET(shared_clean) },
-		{ STR_AND_LEN("Shared_Dirty:"),		SMAP_OFFSET(shared_dirty) },
-		{ STR_AND_LEN("Size:"),			SMAP_OFFSET(size) }
+	struct smap_value {
+		const char *name;
+		ptrdiff_t offset;
 	};
 
-#undef STR_AND_LEN
-#undef SMAP_OFFSET
+	const struct smap_value values[] = {
+		{ "Size:",		SMAP_OFFSET(size) },
+		{ "Rss:",		SMAP_OFFSET(rss) },
+		{ "Shared_Clean:",	SMAP_OFFSET(shared_clean) },
+		{ "Shared_Dirty:",	SMAP_OFFSET(shared_dirty) },
+		{ "Private_Clean:",	SMAP_OFFSET(private_clean) },
+		{ "Private_Dirty:",	SMAP_OFFSET(private_dirty) }
+	};
 
-	struct smap_value* smap;
+	size_t i;
 
-	smap = bsearch(line, values, G_N_ELEMENTS(values), sizeof values[0], compare);
-
-	if (smap) {
+	for (i = 0; i < G_N_ELEMENTS(values); ++i) {
+		char line[80];
 		char *offset;
 		guint64 *value;
 
+		if (!fgets(line, sizeof line, smaps)) {
+			glibtop_warn_io_r(server,
+					  "Could not read smaps value %s",
+					  values[i].name);
+			return;
+		}
+
 		offset = (void*) entry;
-		offset += smap->offset;
+		offset += values[i].offset;
 		value = (void*) offset;
 
-		*value = get_scaled(line + smap->name_len, NULL);
-		return TRUE;
+		*value = get_scaled(line, values[i].name);
 	}
 
-	return is_smap_value(line);
+	entry->flags |= _glibtop_sysdeps_map_entry_smaps;
+
+#undef SMAP_OFFSET
 }
 
 
@@ -159,8 +134,8 @@ glibtop_get_proc_map_s (glibtop *server, glibtop_proc_map *buf,	pid_t pid)
 	FILE *maps;
 	const char *filename;
 	gboolean has_smaps;
-	char *line = NULL;
-	size_t line_size = 0;
+
+	glibtop_init_s (&server, GLIBTOP_SYSDEPS_PROC_MAP, 0);
 
 	memset (buf, 0, sizeof (glibtop_proc_map));
 
@@ -179,32 +154,34 @@ glibtop_get_proc_map_s (glibtop *server, glibtop_proc_map *buf,	pid_t pid)
 
 	while(TRUE)
 	{
-		unsigned long perm;
+		char line[1024];
+
+		unsigned long perm = 0;
+		int rv;
 		guint len;
-		int line_end;
 
 		unsigned short dev_major, dev_minor;
-		guint64 start, end, offset, inode;
+		unsigned long start, end, offset, inode;
 		char flags[4];
-		char *filename;
+		char filename [GLIBTOP_MAP_FILENAME_LEN+1];
 
 		glibtop_map_entry *entry;
 
-		if (getline(&line, &line_size, maps) == -1)
+		if (!fgets(line, sizeof line, maps))
 			break;
 
-	new_entry_line:
+		/* 8 arguments */
+		rv = sscanf(line, PROC_MAPS_FORMAT,
+			    &start, &end, flags, &offset,
+			    &dev_major, &dev_minor, &inode, filename);
 
-		if (sscanf(line, PROC_MAPS_FORMAT,
-			   &start, &end, flags, &offset,
-			   &dev_major, &dev_minor, &inode, &line_end) != 7)
-			continue;
+		if(rv == EOF || rv < 7)
+		  break;
 
-		filename = line + line_end;
-		g_strstrip(filename);
+		if(rv == 7) /* no filename */
+		  filename[0] = '\0';
 
 		/* Compute access permissions. */
-		perm = 0;
 
 		if (flags [0] == 'r')
 			perm |= GLIBTOP_MAP_PERM_READ;
@@ -229,31 +206,19 @@ glibtop_get_proc_map_s (glibtop *server, glibtop_proc_map *buf,	pid_t pid)
 		entry = &g_array_index(entry_list, glibtop_map_entry, len);
 
 		entry->flags = _glibtop_sysdeps_map_entry;
-		entry->start = start;
-		entry->end = end;
-		entry->offset = offset;
-		entry->perm = perm;
-		entry->device = MKDEV(dev_major, dev_minor);
-		entry->inode = inode;
+		entry->start = (guint64) start;
+		entry->end = (guint64) end;
+		entry->offset = (guint64) offset;
+		entry->perm = (guint64) perm;
+		entry->device = (guint64) MKDEV(dev_major, dev_minor);
+		entry->inode = (guint64) inode;
 		g_strlcpy(entry->filename, filename, sizeof entry->filename);
 
-		if (has_smaps) {
-			ssize_t ret;
-			entry->flags |= _glibtop_sysdeps_map_entry_smaps;
+		if (has_smaps)
+			add_smaps(server, maps, entry);
 
-			while ((ret = getline(&line, &line_size, maps)) != -1) {
-				if (!parse_smaps(entry, line))
-					goto new_entry_line;
-			}
-
-			if (ret == -1)
-				goto eof;
-		}
 	}
 
-eof:
-
-	free(line);
 	fclose (maps);
 
 	buf->flags = _glibtop_sysdeps_proc_map;
