@@ -1,4 +1,4 @@
-/* $Id: procsegment.c,v 1.24 2004/06/18 07:24:58 bdejean Exp $ */
+/* $Id: procsegment.c,v 1.27 2005/07/22 19:20:39 bdejean Exp $ */
 
 /* Copyright (C) 1998-99 Martin Baulig
    This file is part of LibGTop 1.0.
@@ -25,6 +25,8 @@
 #include <glibtop/error.h>
 #include <glibtop/procsegment.h>
 
+#include "glibtop_private.h"
+
 static const unsigned long _glibtop_sysdeps_proc_segment =
 (1L << GLIBTOP_PROC_SEGMENT_START_CODE) +
 (1L << GLIBTOP_PROC_SEGMENT_END_CODE) +
@@ -53,7 +55,7 @@ glibtop_get_proc_segment_s (glibtop *server, glibtop_proc_segment *buf,
 			    pid_t pid)
 {
 	char buffer [BUFSIZ], *p;
-	const unsigned pageshift = get_pageshift();
+	const size_t pagesize = get_page_size();
 
 	glibtop_init_s (&server, GLIBTOP_SYSDEPS_PROC_SEGMENT, 0);
 
@@ -86,10 +88,10 @@ glibtop_get_proc_segment_s (glibtop *server, glibtop_proc_segment *buf,
 	buf->data_rss   = strtoull (p, &p, 0);
 	buf->dirty_size = strtoull (p, &p, 0);
 
-	buf->text_rss   <<= pageshift;
-	buf->shlib_rss  <<= pageshift;
-	buf->data_rss   <<= pageshift;
-	buf->dirty_size <<= pageshift;
+	buf->text_rss   *= pagesize;
+	buf->shlib_rss  *= pagesize;
+	buf->data_rss   *= pagesize;
+	buf->dirty_size *= pagesize;
 
 	buf->flags |= _glibtop_sysdeps_proc_segment_statm;
 }
